
/*]  Copyright (c) 2009-2010, Charles McGarvey  [**************************
**]  All rights reserved.
*
* vi:ts=4 sw=4 tw=75
*
* Distributable under the terms and conditions of the 2-clause BSD license;
* see the file COPYING for a complete text of the license.
*
**************************************************************************/

#ifndef _CHARACTER_HH_
#define _CHARACTER_HH_

#include <string>

#include <boost/shared_ptr.hpp>

#include <moof/aabb.hh>
#include <moof/entity.hh>
#include <moof/math.hh>
#include <moof/rigid_body.hh>
#include <moof/sphere.hh>
#include <moof/sprite.hh>

#include "Animation.hh"


class Character;
typedef boost::shared_ptr<Character> CharacterP;


/**
 * Parent class of animate objects with "personalities."  This basically
 * includes the heroine herself and the bad guys.
 */

class Character : public moof::rigid_body2
{
public:

	Character(const std::string& name);
	virtual ~Character() {}

	virtual void update(moof::scalar t, moof::scalar dt);
	virtual void draw(moof::scalar alpha) const;

	void addImpulse(moof::vector2 impulse);
	void addForce(moof::vector2 force);
	void setPosition(moof::vector2 position);

	//virtual int getOctant(const moof::Aabb<3>& aabb) const;

	moof::sprite	tilemap;
	Animation		animation;
};


#endif // _CHARACTER_HH_

